# Copyright (c) 2025 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .constants import SUPPORTED_BACKENDS


def model_name_to_module_name(model_name):
    mod_name = model_name.lower().replace("-", "_").replace(".", "")
    if mod_name[0].isdigit():
        return "m_" + mod_name
    return mod_name


def check_backend(backend):
    if backend not in SUPPORTED_BACKENDS:
        raise ValueError(f"{repr(backend)} is not a supported backend.")
